/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.utils.nsp;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * 
 * @author Jacob
 */
public class PrepareTestPlan {

	public static void main(String[] args) throws Exception {
		if (args.length < 4 || args.length > 10) {
			System.err.println("usage: java " + PrepareTestPlan.class.getName() + " /path/to/testplan /path/to/distribution /path/to/assembled [Thread count] [Hosts configuration file]");
		}
		String testplanFilename = args[0];
		String distributionFilename = args[1];
		String assembledTestplanFilename = args[2];
		String masterThreadCount = "1";
		if (args.length > 3) {
			masterThreadCount = args[3];
		}
		Properties hosts = new Properties();
		if (args.length > 4) {
			String hostsFilename = args[4];
			File hostsFile = new File(hostsFilename);
			if (hostsFile.exists() && hostsFile.canRead()) {
				System.out.println("Loading hosts properties from " + hostsFilename);
				hosts.load(new FileInputStream(hostsFilename));
			} else {
				System.err.println("File not found: " + hostsFilename);
				System.exit(1);
			}
		}
		String nspHost = hosts.getProperty("nsp.host", "localhost");
		String nspPort = hosts.getProperty("nsp.port", "80");
		String stsHost = hosts.getProperty("sts.host", nspHost);
		String stsPort = hosts.getProperty("sts.port", nspPort);
		String ocesHost = hosts.getProperty("oces.host", stsHost);
		String ocesPort = hosts.getProperty("oces.port", stsPort);

		System.out.println("Assembling testplan template " + testplanFilename + " with distribution " + distributionFilename + " on NSP server " + nspHost + ":" + nspPort + ", STS server " + stsHost + ":" + stsPort + " and OCES server " + ocesHost + ":" + ocesPort + " to testplan " + assembledTestplanFilename);

		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		Document testplan = documentBuilderFactory.newDocumentBuilder().parse(new FileInputStream(new File(testplanFilename)));
		Document distribution = documentBuilderFactory.newDocumentBuilder().parse(new FileInputStream(new File(distributionFilename)));

		XPathFactory xPathFactory = XPathFactory.newInstance();
		XPathExpression argumentsExpression = xPathFactory.newXPath().compile("//Arguments[@testname=\"Distribution\"]");
		XPathExpression masterThreadCountExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"MASTER_THREAD_COUNT\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression nspHostExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"NSP_HOSTNAME\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression nspPortExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"NSP_PORT\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression stsHostExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"STS_HOSTNAME\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression stsPortExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"STS_PORT\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression ocesHostExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"OCES_MOCK_HOSTNAME\"]/stringProp[@name=\"Argument.value\"]");
		XPathExpression ocesPortExpression = xPathFactory.newXPath().compile("//elementProp[@name=\"OCES_MOCK_PORT\"]/stringProp[@name=\"Argument.value\"]");

		// Replace "Distribution" element with selected distribution
		Element testplanArguments = (Element) argumentsExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		Element distributionArguments = (Element) argumentsExpression.evaluate(distribution.getDocumentElement(), XPathConstants.NODE);
		Node testplanCollectionProp = testplanArguments.getElementsByTagName("collectionProp").item(0);
		Node distributionCollectionProp = distributionArguments.getElementsByTagName("collectionProp").item(0);
		Node collectionProp = testplan.importNode(distributionCollectionProp, true);
		testplanArguments.replaceChild(collectionProp, testplanCollectionProp);

		// Update the master thread count in the distribution
		Element masterThreadCountProperty = (Element) masterThreadCountExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		masterThreadCountProperty.setTextContent(masterThreadCount);

		// Update NSP host properties
		Element nspHostProperty = (Element) nspHostExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		nspHostProperty.setTextContent(nspHost);
		Element nspPortProperty = (Element) nspPortExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		nspPortProperty.setTextContent(nspPort);

		// Update STS host properties
		Element stsHostProperty = (Element) stsHostExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		stsHostProperty.setTextContent(stsHost);
		Element stsPortProperty = (Element) stsPortExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		stsPortProperty.setTextContent(stsPort);

		// Update OCES Mock host properties
		Element ocesHostProperty = (Element) ocesHostExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		ocesHostProperty.setTextContent(ocesHost);
		Element ocesPortProperty = (Element) ocesPortExpression.evaluate(testplan.getDocumentElement(), XPathConstants.NODE);
		ocesPortProperty.setTextContent(ocesPort);

		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(testplan);
		StreamResult assembledTestplan = new StreamResult(new File(assembledTestplanFilename));
		transformer.transform(source, assembledTestplan);
	}

}
