package com.arosii.jmeter.protocol.ihe.control.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.xml.bind.annotation.XmlEnumValue;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.util.JMeterUtils;

import com.arosii.jmeter.protocol.ihe.sampler.AbstractIHESampler;
import com.arosii.jmeter.protocol.nsp.control.gui.AbstractNSPSamplerGui;
import com.arosii.jmeter.resources.ihe.Messages;

import dk.nsi.hsuid._2016._08.hsuid_1_1.SubjectIdentifierType;

public abstract class AbstractIHESamplerGui extends AbstractNSPSamplerGui {

	private static final long serialVersionUID = 1L;

	private JCheckBox dgws;

	private JTextField cpr;

	private JTextField organisationIdentifier;

	private JComboBox<String> organisationIdentifierFormat;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getStaticLabel() {
		String label = Messages.getResString(getLabelResource());
		if (label == null || label.startsWith(JMeterUtils.RES_KEY_PFX)) {
			label = super.getStaticLabel();
		}
		return label;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		cpr.setText(element.getPropertyAsString(AbstractIHESampler.CITIZEN_CPR_NUMBER, ""));
		organisationIdentifier.setText(element.getPropertyAsString(AbstractIHESampler.ORGANISATION_IDENTIFIER, ""));
		organisationIdentifierFormat.setSelectedItem(element.getPropertyAsString(AbstractIHESampler.ORGANISATION_IDENTIFIER_FORMAT, getDefaultOrganisationIdentifierFormat()));
		dgws.setSelected(element.getPropertyAsBoolean(AbstractIHESampler.ADD_DGWS_HEADERS, false));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(AbstractIHESampler.CITIZEN_CPR_NUMBER, cpr.getText());
		element.setProperty(AbstractIHESampler.ORGANISATION_IDENTIFIER, organisationIdentifier.getText());
		element.setProperty(AbstractIHESampler.ORGANISATION_IDENTIFIER_FORMAT, (String) organisationIdentifierFormat.getSelectedItem());
		element.setProperty(AbstractIHESampler.ADD_DGWS_HEADERS, dgws.isSelected());
	}

	@Override
	public void clearGui() {
		super.clearGui();
		cpr.setText("");
		organisationIdentifier.setText("");
		organisationIdentifierFormat.setSelectedItem(getDefaultOrganisationIdentifierFormat());
	}

	protected void appendMessageHeadersPanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_headers_panel")));
		panel.add(getAddHeadersPanel());
		panel.add(getCPRPanel());
		panel.add(getOrganisationIdentifierPanel());
		panel.add(getOrganisationIdentifierFormatPanel());
		configurationPanel.add(panel);
	}

	private JPanel getAddHeadersPanel() {
		dgws = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("ihe_headers_dgws"));

		JPanel dgwsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		dgwsPanel.add(dgws);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(dgwsPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getCPRPanel() {
		cpr = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_headers_cpr"));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getOrganisationIdentifierPanel() {
		organisationIdentifier = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_headers_org_code"));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(organisationIdentifier);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getOrganisationIdentifierFormatPanel() {
		organisationIdentifierFormat = new JComboBox<String>(getOrganisationIdentifierFormats());
		JLabel label = new JLabel(Messages.getResString("ihe_headers_org_format"));

		JPanel organisationIdentifierFormatPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		organisationIdentifierFormatPanel.add(organisationIdentifierFormat);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(organisationIdentifierFormatPanel, BorderLayout.CENTER);

		return panel;
	}

	private String[] getOrganisationIdentifierFormats() {
		SubjectIdentifierType[] subjectIdentifierTypes = SubjectIdentifierType.values();
		List<String> formats = new ArrayList<String>(subjectIdentifierTypes.length);
		try {
			for (SubjectIdentifierType subjectIdentifierType : subjectIdentifierTypes) {
				formats.add(SubjectIdentifierType.class.getField(subjectIdentifierType.name()).getAnnotation(XmlEnumValue.class).value());
			}
		} catch (Exception e) {
			throw new IllegalStateException(e);
		}
		return (String[]) formats.toArray(new String[formats.size()]);
	}

	private String getDefaultOrganisationIdentifierFormat() {
		try {
			return SubjectIdentifierType.class.getField(SubjectIdentifierType.NSI_SORCODE.name()).getAnnotation(XmlEnumValue.class).value();
		} catch (Exception e) {
			throw new IllegalStateException(e);
		}
	}
}
