package com.arosii.jmeter.protocol.ihe.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.ihe.sampler.ITI43Sampler;
import com.arosii.jmeter.resources.ihe.Messages;

public class ITI43SamplerGUI extends AbstractIHESamplerGui {

	private JCheckBox extractFromQuery;

	private JTextField repositoryUniqueId;

	private JTextField documentUniqueId;

	@Override
	public String getLabelResource() {
		return "ihe_iti43_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ITI43Sampler sampler = new ITI43Sampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		extractFromQuery.setSelected(element.getPropertyAsBoolean(ITI43Sampler.FROM_EXTRACTOR, false));
		if (extractFromQuery.isSelected()) {
			repositoryUniqueId.setText("");
			documentUniqueId.setText("");
		} else {
			repositoryUniqueId.setText(element.getPropertyAsString(ITI43Sampler.REPOSITORY_UNIQUE_ID, ""));
			documentUniqueId.setText(element.getPropertyAsString(ITI43Sampler.DOCUMENT_UNIQUE_ID, ""));
		}
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ITI43Sampler.FROM_EXTRACTOR, extractFromQuery.isSelected());
		if (!extractFromQuery.isSelected()) {
			element.setProperty(ITI43Sampler.REPOSITORY_UNIQUE_ID, repositoryUniqueId.getText());
			element.setProperty(ITI43Sampler.DOCUMENT_UNIQUE_ID, documentUniqueId.getText());
		} else {
			element.removeProperty(ITI43Sampler.REPOSITORY_UNIQUE_ID);
			element.removeProperty(ITI43Sampler.DOCUMENT_UNIQUE_ID);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendMessageHeadersPanel(configurationPanel);
		appendDocumentPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		updateFields();
	}

	private void appendDocumentPanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti43_panel")));
		panel.add(getExtractFromQueryPanel());
		panel.add(getRepositoryUniqueIdPanel());
		panel.add(getDocumentUniqueIdPanel());
		configurationPanel.add(panel);
	}

	private JPanel getExtractFromQueryPanel() {
		extractFromQuery = new JCheckBox();
		extractFromQuery.addActionListener((e -> updateFields()));
		JLabel label = new JLabel(Messages.getResString("ihe_iti43_extract_from_query"));

		JPanel extractFromQueryPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		extractFromQueryPanel.add(extractFromQuery);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(extractFromQueryPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRepositoryUniqueIdPanel() {
		repositoryUniqueId = new JTextField(80);
		JLabel label = new JLabel(Messages.getResString("ihe_iti43_repository_unique_id"));

		JPanel repositoryUniqueIdPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		repositoryUniqueIdPanel.add(repositoryUniqueId);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(repositoryUniqueIdPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getDocumentUniqueIdPanel() {
		documentUniqueId = new JTextField(80);
		JLabel label = new JLabel(Messages.getResString("ihe_iti43_document_unique_id"));

		JPanel documentUniqueIdPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		documentUniqueIdPanel.add(documentUniqueId);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(documentUniqueIdPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		if (extractFromQuery.isSelected()) {
			repositoryUniqueId.setEnabled(false);
			documentUniqueId.setEnabled(false);
			repositoryUniqueId.setText("");
			documentUniqueId.setText("");
		} else {
			repositoryUniqueId.setEnabled(true);
			documentUniqueId.setEnabled(true);
		}

	}
}
