package com.arosii.jmeter.protocol.ihe.sampler;

import java.net.HttpURLConnection;
import java.util.Arrays;

import org.apache.jmeter.samplers.SampleResult;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLAdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLQueryResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.QueryReturnType;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.QueryRegistryTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.QueryResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;

public class ITI18Sampler extends AbstractIHEDocumentRegistrySampler<Iti18PortType> {

	@Override
	protected SampleResult sample() throws Exception {
		ITI18SampleResult res = new ITI18SampleResult();
		res.setURL(getUrl());
		res.setHTTPMethod("ITI18");
		Iti18PortType iti18Client = getClient(Iti18PortType.class);
		QueryRegistry queryRegistry = buildQueryRegistry();
		res.setQueryRegistry(queryRegistry);

		AdhocQueryRequest adhocQueryRequest = marshall(queryRegistry);
		res.sampleStart();
		AdhocQueryResponse adhocQueryResponse = iti18Client.documentRegistryRegistryStoredQuery(adhocQueryRequest);
		res.sampleEnd();
		QueryResponse queryResponse = unmarshall(adhocQueryResponse);
		res.setQueryResponse(queryResponse);

		res.setSamplerData(getContentCollectorOutputStream().content());
		res.setResponseData(getContentCollectorInputStream().content(), "utf-8");

		res.setResponseCode(Integer.toString(HttpURLConnection.HTTP_OK));
		res.setSuccessful(queryResponse.getStatus().equals(Status.SUCCESS));
		res.setResponseMessage(queryResponse.getStatus().getOpcode30());
		res.setDataEncoding("utf-8");
		res.setContentType("text/xml");
		return res;
	}

	private QueryRegistry buildQueryRegistry() {
		FindDocumentsQuery findDocumentsQuery = new FindDocumentsQuery();
		// Patient ID
		findDocumentsQuery.setPatientId(new Identifiable(getCitizenCPRNumber(), new AssigningAuthority("1.2.208.176.1.2")));

		// Availability Status
		findDocumentsQuery.setStatus(Arrays.asList(AvailabilityStatus.APPROVED));

		// Søg efter både stable og on-demand
		findDocumentsQuery.setDocumentEntryTypes(Arrays.asList(DocumentEntryType.ON_DEMAND, DocumentEntryType.STABLE));

		// Fjern måske?
		//findDocumentsQuery.setTypeCodes(Arrays.asList(new Code("39289-4", null, "2.16.840.1.113883.6.1")));

		QueryRegistry queryRegistry = new QueryRegistry(findDocumentsQuery);
		queryRegistry.setReturnType(QueryReturnType.LEAF_CLASS);

		return queryRegistry;
	}

	private static AdhocQueryRequest marshall(QueryRegistry queryRegistry) {
		QueryRegistryTransformer queryRegistryTransformer = new QueryRegistryTransformer();
		EbXMLAdhocQueryRequest ebxmlAdhocQueryRequest = queryRegistryTransformer.toEbXML(queryRegistry);
		AdhocQueryRequest adhocQueryRequest = (AdhocQueryRequest) ebxmlAdhocQueryRequest.getInternal();
		return adhocQueryRequest;
	}

	private static QueryResponse unmarshall(AdhocQueryResponse adhocQueryResponse) {
		QueryResponseTransformer queryResponseTransformer = new QueryResponseTransformer(new EbXMLFactory30());
		EbXMLQueryResponse30 ebXmlresponse = new EbXMLQueryResponse30(adhocQueryResponse);
		QueryResponse queryResponse = queryResponseTransformer.fromEbXML(ebXmlresponse);
		return queryResponse;
	}

}
