package com.arosii.jmeter.protocol.ihe.sampler;

import java.net.HttpURLConnection;
import java.util.Collection;

import org.apache.jmeter.protocol.http.sampler.HTTPSamplerBase;
import org.apache.jmeter.samplers.SampleResult;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLRetrieveDocumentSetResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.RetrieveDocumentSetRequestTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.RetrieveDocumentSetResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;

public class ITI43Sampler extends AbstractIHEDocumentRepositorySampler<Iti43PortType> {

	public static final String FROM_EXTRACTOR = "ITI43Sampler.from_extractor";

	public static final String REPOSITORY_UNIQUE_ID = "ITI43Sampler.repository_unique_id";

	public static final String DOCUMENT_UNIQUE_ID = "ITI43Sampler.document_unique_id";

	public ITI43Sampler() {
		setProperty(HTTPSamplerBase.PATH, "/ddsrepository/service");
	}

	@Override
	protected SampleResult sample() throws Exception {
		ITI43SampleResult res = new ITI43SampleResult();
		res.setURL(getUrl());
		res.setHTTPMethod("ITI18");
		Iti43PortType iti43Client = getClient(Iti43PortType.class);
		RetrieveDocumentSet retrieveDocumentSet = buildRetrieveDocumentSet();
		res.setRetrieveDocumentSet(retrieveDocumentSet);

		RetrieveDocumentSetRequestType retrieveDocumentSetRequestType = marshall(retrieveDocumentSet);
		res.sampleStart();
		RetrieveDocumentSetResponseType retrieveDocumentSetResponseType = iti43Client.documentRepositoryRetrieveDocumentSet(retrieveDocumentSetRequestType);
		res.sampleEnd();
		RetrievedDocumentSet retrievedDocumentSet = unmarshall(retrieveDocumentSetResponseType);
		res.setRetrievedDocumentSet(retrievedDocumentSet);

		res.setSamplerData(getContentCollectorOutputStream().content());
		res.setResponseData(getContentCollectorInputStream().content(), "utf-8");

		res.setResponseCode(Integer.toString(HttpURLConnection.HTTP_OK));
		res.setSuccessful(retrievedDocumentSet.getStatus().equals(Status.SUCCESS));
		res.setResponseMessage(retrievedDocumentSet.getStatus().getOpcode30());
		res.setDataEncoding("utf-8");
		res.setContentType("text/xml");

		return res;
	}

	@SuppressWarnings("unchecked")
	private RetrieveDocumentSet buildRetrieveDocumentSet() {
		RetrieveDocumentSet retrieveDocumentSet = new RetrieveDocumentSet();
		if (getPropertyAsBoolean(FROM_EXTRACTOR, false)) {
			Collection<DocumentEntry> documentEntries = (Collection<DocumentEntry>) getThreadContext().getVariables().getObject(DocumentEntry.class.getName());
			for (DocumentEntry documentEntry : documentEntries) {
				RetrieveDocument retrieveDocument = new RetrieveDocument();
				retrieveDocument.setRepositoryUniqueId(documentEntry.getRepositoryUniqueId());
				retrieveDocument.setDocumentUniqueId(documentEntry.getUniqueId());
				retrieveDocumentSet.getDocuments().add(retrieveDocument);
			}
		} else {
			RetrieveDocument retrieveDocument = new RetrieveDocument();
			retrieveDocument.setRepositoryUniqueId(getPropertyAsString(REPOSITORY_UNIQUE_ID));
			retrieveDocument.setDocumentUniqueId(getPropertyAsString(DOCUMENT_UNIQUE_ID));
			retrieveDocumentSet.getDocuments().add(retrieveDocument);
		}
		return retrieveDocumentSet;
	}

	private static RetrieveDocumentSetRequestType marshall(RetrieveDocumentSet retrieveDocumentSet) {
		RetrieveDocumentSetRequestTransformer transformer = new RetrieveDocumentSetRequestTransformer(new EbXMLFactory30());
		EbXMLRetrieveDocumentSetRequest ebXMLRequest = transformer.toEbXML(retrieveDocumentSet);
		RetrieveDocumentSetRequestType retrieveDocumentSetRequestType = (RetrieveDocumentSetRequestType) ebXMLRequest.getInternal();
		return retrieveDocumentSetRequestType;
	}

	private static RetrievedDocumentSet unmarshall(RetrieveDocumentSetResponseType retrieveDocumentSetResponseType) {
		RetrieveDocumentSetResponseTransformer transformer = new RetrieveDocumentSetResponseTransformer(new EbXMLFactory30());
		EbXMLRetrieveDocumentSetResponse ebXmlResponse = new EbXMLRetrieveDocumentSetResponse30(retrieveDocumentSetResponseType);
		RetrievedDocumentSet retrievedDocumentSet = transformer.fromEbXML(ebXmlResponse);
		return retrievedDocumentSet;
	}
}
