package com.arosii.jmeter.protocol.ihe.visualizers;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.apache.jmeter.visualizers.RequestView;
import org.apache.jorphan.gui.GuiUtils;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.Query;

import com.arosii.jmeter.protocol.ihe.sampler.IHESampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI18SampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI43SampleResult;
import com.arosii.jmeter.resources.ihe.Messages;

public class RequestViewIHE implements RequestView {

	private JTextArea sampleDataField;

	private JPanel panel;

	@Override
	public void init() {
		panel = new JPanel(new BorderLayout(0, 5));
		sampleDataField = new JTextArea();
		sampleDataField.setEditable(false);
		sampleDataField.setLineWrap(true);
		sampleDataField.setWrapStyleWord(true);

		panel.add(GuiUtils.makeScrollPane(sampleDataField));
	}

	@Override
	public void clearData() {
		sampleDataField.setText("");
	}

	@Override
	public void setSamplerResult(Object userObject) {
		StringBuilder iheText = new StringBuilder();
		if (userObject instanceof IHESampleResult) {
			IHESampleResult sampleResult = (IHESampleResult) userObject;
			if (sampleResult instanceof ITI18SampleResult) {
				iheText.append("ITI18").append('\n');
				ITI18SampleResult iti18SampleResult = (ITI18SampleResult) sampleResult;
				QueryRegistry queryRegistry = iti18SampleResult.getQueryRegistry();
				if (queryRegistry != null) {
					Query query = queryRegistry.getQuery();
					if (query instanceof FindDocumentsQuery) {
						FindDocumentsQuery findDocumentsQuery = (FindDocumentsQuery) query;
						iheText.append(findDocumentsQuery.getPatientId().getId()).append('\n');
						iheText.append('\n');
						for (AvailabilityStatus availabilityStatus : findDocumentsQuery.getStatus()) {
							iheText.append(availabilityStatus.toString()).append('\n');
						}
						iheText.append('\n');
						for (DocumentEntryType documentEntryType : findDocumentsQuery.getDocumentEntryTypes()) {
							iheText.append(documentEntryType.toString()).append('\n');
						}
					}
				} else {
					iheText.append("NULL");
				}
			} else if (sampleResult instanceof ITI43SampleResult) {
				iheText.append("ITI43").append('\n');
				ITI43SampleResult iti43SampleResult = (ITI43SampleResult) sampleResult;
				RetrieveDocumentSet retrieveDocumentSet = iti43SampleResult.getRetrieveDocumentSet();
				if (retrieveDocumentSet != null) {
					for (RetrieveDocument retrieveDocument : retrieveDocumentSet.getDocuments()) {
						iheText.append('\n');
						iheText.append(retrieveDocument.getRepositoryUniqueId()).append('\n');
						iheText.append(retrieveDocument.getDocumentUniqueId()).append('\n');
					}
				} else {
					iheText.append("NULL");
				}
			}
		}
		sampleDataField.setText(iheText.toString());
	}

	@Override
	public JPanel getPanel() {
		return panel;
	}

	@Override
	public String getLabel() {
		return Messages.getResString("view_results_table_request_tab_ihe");
	}

}
